// v1.0 31.12.2000
// Gunnar Prien Osterende 35 27389 Helvesiek
// Gunnar@Prien-ROW.DE

unit GPExcelKomponente;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, Excel97;

TYPE
  TSetCellProperty  = (cpSetFont, cpSetFontColor, cpSetColor, cpSetBorderColor);
  TSetCellPropertys = set of TSetCellProperty;

  TGPExcelBeforeClose = procedure(Sender: TObject) of object;

type
  TGPCustomExcel = class(TComponent)
   private
    FOnBeforeClose : TGPExcelBeforeClose;

    FExcelApp        : TExcelApplication;
    FApp             : Variant;
    FSheet           : Variant;
    FActive          : boolean;
    FVisible         : boolean;
    FFileName        : string;
    FCaption         : string;
    FDefaultSheetName: string;
    FX0, FY0         : integer;
    FSheetName       : string;
    FSheetIndex      : integer;
    FIsScrUpdate     : boolean;
    FSetCellProperty : TSetCellPropertys;
    FFont            : TFont;
    FBorderColor     : TColor;
    FColor           : TColor;

    procedure SetActive(const Value: boolean);
    function  GetFileName: string;
    procedure SetFileName(const Value: string);
    procedure SetVisible(const Value: boolean);
    function  GetCellName(X, Y: integer): string;
    function  GetCellByName(aName: string; X,Y: integer): string;
    procedure SetCellbyName(aName: string; X,Y: integer; const Value: string);
    procedure SheetByName(aName: string);
    procedure SheetByIndex(aIndex: integer);
    function  GetWorkSheetCount: integer;
    function  GetX(aX: integer): integer;
    function  GetY(aY: integer): integer;
    function  GetApp: Variant;
    function  GetWorkSheetName(ii: integer): string;
    procedure SetWorkSheetName(ii: integer; const Value: string);
    function  GetHasChanged: boolean;
    function  GetScreenUpdate: boolean;
    procedure SetScreenUpdate(const Value: boolean);
    function  GetCell(X, Y: integer): string;
    procedure SetCell(X, Y: integer; const Value: string);
    procedure SetFont(const Value: TFont);
    function  GetActive: boolean;
    function  GetSetCellProperty: TSetCellPropertys;
    function  GetVisible: boolean;
    procedure SetSetCellProperty(const Value: TSetCellPropertys);
    function  GetFont: TFont;
    function  GetColor: TColor;
    procedure SetColor(const Value: TColor);
    function  GetBorderColor: TColor;
    procedure SetBorderColor(const Value: TColor);
    procedure SetHasChanged(const Value: boolean);
    procedure ExternalClose(Sender: TObject; var Wb: OleVariant;
                                             var Cancel: OleVariant);
    function  GetCaption: string;
    procedure SetCaption(const Value: string);
    function  GetCellWidth(X: integer): integer;
    procedure SetCellWidth(X: integer; const Value: integer);
    function  GetCellHeight(Y: integer): integer;
    procedure SetCellHeight(Y: integer; const Value: integer);
    function  GetDefaultSheetName: string;
    procedure SetDefaultSheetName(const Value: string);

   protected
    property Active: boolean read GetActive write SetActive default false;
    property Visible: boolean read GetVisible write SetVisible default true;
    property Filename: string read GetFileName write SetFileName;
    property X0: integer read FX0 write FX0 default 1;
    property Y0: integer read FY0 write FY0 default 1;

    property ActiveCellProperty: TSetCellPropertys
                                 read  GetSetCellProperty
                                 write SetSetCellProperty default [];

    property Font: TFont read GetFont write SetFont;

    property Color: TColor read GetColor write SetColor default clWindow;
    property BorderColor: TColor read GetBorderColor write SetBorderColor default clSilver;

    property HasChanged: boolean read GetHasChanged write SetHasChanged default false;

    property Caption: string read GetCaption write SetCaption;

    property DefaultSheetName: string read  GetDefaultSheetName
                                      write SetDefaultSheetName;

    property OnBeforeClose: TGPExcelBeforeClose read FOnBeforeClose write FOnBeforeClose;

    property Excel: Variant read GetApp;
    property ExcelApplication: TExcelApplication Read FExcelApp;

    property CellName[X,Y: integer]: string read GetCellName;
    property SheetCell[aSheetName: string; X,Y: integer]: string read GetCellByName write SetCellbyName;
    property Cell[X,Y: integer]: string read GetCell write SetCell;
    property CellWidth[X: integer]: integer read GetCellWidth write SetCellWidth;
    property CellHeight[Y: integer]: integer read GetCellHeight write SetCellHeight;
    property WorkSheetCount: integer read GetWorkSheetCount;
    property WorkSheetName[ii: integer]: string read  GetWorkSheetName
                                                write SetWorkSheetName;

    property ScreenUpdate: boolean read  GetScreenUpdate
                                   write SetScreenUpdate;


   public
    constructor Create(AOwner: TComponent); override;
    destructor  destroy; override;

    procedure LoadFromFile(aFileName: string);
    procedure SaveToFile(aFileName: string);

    procedure CellPropertysToArea(X1,Y1, X2,Y2: integer);
    procedure CellPropertysFromCell(X,Y: integer);

   published
  end;

type
  TGPExcel = class(TGPCustomExcel)
   private
   protected
   public
    property Excel;
    property ExcelApplication;
    property CellName;
    property SheetCell;
    property Cell;
    property CellWidth;
    property CellHeight;
    property WorkSheetCount;
    property WorkSheetName;
    property ScreenUpdate;

   published
    property Active;
    property Visible;
    property Filename;
    property X0;
    property Y0;
    property ActiveCellProperty;
    property Font;
    property Color;
    property BorderColor;
    property HasChanged;
    property Caption;
    property DefaultSheetName;
    property OnBeforeClose;

  end;

implementation

constructor TGPCustomExcel.Create(AOwner: TComponent);
begin
  inherited;

  FX0               := 1;
  FY0               := 1;
  FVisible          := TRUE;
  FColor            := clWindow;
  FBorderColor      := clSilver;
  FFont             := TFont.Create;
  FCaption          := '';
  FDefaultSheetName := 'Data';
end; { Create }

{ ----- }

destructor TGPCustomExcel.destroy;
begin
  Active := false;
  FreeAndNil(FFont);
  inherited;
end; { destroy }

{ ----- }

function TGPCustomExcel.GetCell(X, Y: integer): string;
begin
  IF Active THEN
  BEGIN
    Result := FSheet.Cells[GetY(Y), GetX(X)];
  END;
end; { GetCell }

{ ----- }

procedure TGPCustomExcel.SetCell(X, Y: integer; const Value: string);
begin
  IF Active THEN
  BEGIN
    CellPropertysToArea(X,Y, X,Y);
    FSheet.Cells[GetY(Y), GetX(X)] := Value;
  END;
end; { SetCell }

{ ----- }

function TGPCustomExcel.GetCellByName(aName: string; X,Y: integer): string;
begin
  SheetByName(aName);
  Result := Cell[X,Y];
end; { GetCellByName }

{ ----- }

procedure TGPCustomExcel.SetCellByName(aName: string; X,Y: integer; const Value: string);
begin
  SheetByName(aName);
  Cell[X, Y] := Value;
end; { SetCellByName }

{ ----- }

function TGPCustomExcel.GetCellName(X, Y: integer): string;
begin
  X := GetX(X) - 1;
  Result := CHR(ORD('A') + X MOD 26) + IntToStr(GetY(Y));
  IF X >= 26
  THEN Result := CHR(ORD('A') + X DIV 26 - 1) + Result;
end; { GetCellName }

{ ----- }

procedure TGPCustomExcel.LoadFromFile(aFileName: string);
begin
  Excel.Workbooks.Open(aFileName);
  FFileName := aFileName;
end; { LoadFromFile }

{ ----- }

procedure TGPCustomExcel.SaveToFile(aFileName: string);
begin
  Excel.WorkBooks[1].SaveAs(aFileName);
  FFileName := aFileName;
end; { SaveToFile }

{ ----- }

procedure TGPCustomExcel.SetActive(const Value: boolean);
var
  aIndex : integer;

begin
  IF Value <> FActive THEN
  BEGIN
    IF Value THEN
    BEGIN              { Sanduhr zeigen, da es beim 1. mal etwas lnger dauert }
      Screen.Cursor := crHourGlass;
      TRY
        FExcelApp := TExcelApplication.Create(NIL);
        TRY
          ExcelApplication.OnWorkbookBeforeClose := ExternalClose;
          ExcelApplication.Connect;     { CreateOleObject('Excel.Application') }
          FApp := ExcelApplication.Application;

          IF Visible
          THEN FApp.Visible := True;

          FApp.ScreenUpdating := TRUE;
          FIsScrUpdate        := TRUE;

          IF (FFileName = '')
               OR
             (NOT FileExists(FFileName)) THEN
          BEGIN
            FApp.Workbooks.Add(xlWBatWorkSheet);
            FApp.Workbooks[1].WorkSheets[1].Name := DefaultSheetName;
          END
          ELSE
            FApp.Workbooks.Open(FFileName);

          aIndex := 1;
          FSheet := FApp.Workbooks[1].WorkSheets[aIndex];

          IF FCaption = ''
          THEN FCaption := FApp.Caption
          ELSE FApp.Caption := FCaption;

        EXCEPT
          IF NOT VarIsEmpty(FApp) THEN
          BEGIN
            FApp.DisplayAlerts := false;   { bei genderten Daten nicht fragen }
            FApp.Quit;                                { Excel wieder schlieen }
            FApp   := Unassigned;                   { Variante nun unbrauchbar }
            FSheet := Unassigned;
          END;

          FreeAndNil(FExcelApp);
          Raise;               { Diese Exception an den Aufrufer weiterreichen }
        END;

      FINALLY
        Screen.Cursor := crDefault;
      END;
    END
    ELSE
    BEGIN
      FSheetName := '';
      FSheetIndex := 0;
      FIsScrUpdate := FALSE;
      IF NOT VarIsEmpty(FApp) THEN
      BEGIN
        FApp.DisplayAlerts := false;
        FApp.Quit;
        FApp   := Unassigned;
        FSheet := Unassigned;
      END;
      FreeAndNil(FExcelApp);
    END;

    FActive := Value;
  END;
end; { SetActive }

{ ----- }

function TGPCustomExcel.GetFileName: string;
begin
  IF (FFileName = '') AND Active THEN
  BEGIN
    FFileName := FApp.Workbooks[1].Fullname;
  END;

  Result := FFileName;
end; { GetFileName }

{ ----- }

procedure TGPCustomExcel.SetFileName(const Value: string);
begin
  IF Active
  THEN LoadFromFile(Value);
  FFileName := Value;
end; { SetFileName }

{ ----- }

procedure TGPCustomExcel.SetVisible(const Value: boolean);
begin
  IF Active AND (NOT Value)
  THEN   { Ein sichbares Excel lt sich nicht verstecken ! }
  ELSE
  BEGIN
    FVisible := Value;
    IF Active
    THEN Excel.Visible := TRUE;
  END;
end; { SetVisible }

{ ----- }

procedure TGPCustomExcel.SheetByIndex(aIndex: integer);
var
  WCount : integer;

begin
  IF aIndex <> FSheetIndex THEN
  BEGIN
    IF aIndex < 1
    THEN Raise Exception.Create('SheetByIndex: Index unterschreitet untere Schranke');
    WCount := WorkSheetCount;
    IF aIndex > (WCount + 1)
    THEN Raise Exception.Create('SheetByIndex: Index berschreitet obere Schranke');
    IF aIndex > WCount THEN
    BEGIN
      FSheet := Excel.Workbooks[1].WorkSheets[WCount];
      Excel.Workbooks[1].WorkSheets.Add(,FSheet);
      Excel.Workbooks[1].WorkSheets[aIndex].Name := 'Index ' + IntToStr(WCount + 1);
    END;

    FSheet      := Excel.Workbooks[1].WorkSheets[aIndex];
    FSheetIndex := aIndex;
    FSheetName  := FSheet.Name;
    FSheet.Activate;
  END;
end; { SheetByIndex }

{ ----- }

procedure TGPCustomExcel.SheetByName(aName: string);
var
  ii    : integer;
  WCount: integer;

begin
  IF aName <> FSheetName THEN
  BEGIN
    WCount := WorkSheetCount;
    for ii := 1 to WCount do
    begin
      IF aName = Excel.Workbooks[1].WorkSheets[ii].Name THEN
      BEGIN
        FSheet := Excel.Workbooks[1].WorkSheets[ii];
        FSheetIndex := ii;
        FSheetName  := aName;
        FSheet.Activate;
        Exit;
      END;
    end;

    SheetByIndex(WCount + 1);
    FSheet.Name := aName;
  END;
end; { SheetByName }

{ ----- }

function TGPCustomExcel.GetWorkSheetCount: integer;
begin
  IF Active THEN
  BEGIN
    TRY
      Result := Excel.Workbooks[1].WorkSheets.Count;
    EXCEPT
      Result := 0;
    END;
  END
  ELSE
    Result := -1;
end; { GetWorkSheetCount }

{ ----- }

function TGPCustomExcel.GetX(aX: integer): integer;
begin
  IF aX >= X0
  THEN Result := aX - X0 + 1
  ELSE Raise Exception.Create('GetX: X unterschreitet untere Schranke');
end; { GetX }

{ ----- }

function TGPCustomExcel.GetY(aY: integer): integer;
begin
  IF aY >= Y0
  THEN Result := aY - Y0 + 1
  ELSE Raise Exception.Create('GetY: Y unterschreitet untere Schranke');
end; { GetY }

{ ----- }

function TGPCustomExcel.GetApp: Variant;
begin
  IF NOT Active THEN
  BEGIN
    Visible := TRUE;
    Active  := TRUE;
  END;

  Result := FApp;
end; { GetApp }

{ ----- }

function TGPCustomExcel.GetWorkSheetName(ii: integer): string;
begin
  Result := '';
  IF (ii >= 1) AND (ii <= WorkSheetCount) THEN
  BEGIN
    SheetByIndex(ii);
    Result := FSheetName;
  END;
end; { GetWorkSheetName }

{ ----- }

procedure TGPCustomExcel.SetWorkSheetName(ii: integer; const Value: string);
begin
  IF (ii >= 1) AND (ii <= WorkSheetCount) THEN
  BEGIN
    SheetByIndex(ii);
    FSheet.Name:= Value;
    FSheetName := Value;
  END
  ELSE
    IF ii = (WorkSheetCount + 1) THEN
    BEGIN
      SheetByName(Value);                   { ggf. ein neues Worksheet anlegen }
    END;
end; { SetWorkSheetName }

{ ----- }

function TGPCustomExcel.GetHasChanged: boolean;
begin
  Result := false;

  IF Active
  THEN Result := NOT Excel.Workbooks[1].Saved;
end; { GetHasChanged }

{ ----- }

procedure TGPCustomExcel.SetHasChanged(const Value: boolean);
begin
  IF Active
  THEN Excel.Workbooks[1].Saved := NOT Value;
end; { SetHasChanged }

{ ----- }

function TGPCustomExcel.GetScreenUpdate: boolean;
begin
  Result := FIsScrUpdate;
end; { GetScreenUpdate }

{ ----- }

procedure TGPCustomExcel.SetScreenUpdate(const Value: boolean);
begin
  IF Active THEN
  BEGIN
    Excel.ScreenUpdating := Value;
    FIsScrUpdate := Value;
  END;
end; { SetScreenUpdate }

{ ----- }

procedure TGPCustomExcel.SetFont(const Value: TFont);
begin
  IF Value <> NIL THEN
  BEGIN
    IF NOT Assigned(FFont) THEN
    BEGIN
      FFont := TFont.Create;
    END;
    FFont.Assign(Value);
  END
  ELSE
    FreeAndNil(FFont);
end; { SetFont }

{ ----- }

procedure TGPCustomExcel.CellPropertysToArea(X1, Y1, X2, Y2: integer);
var
  Range     : Variant;
  ExcelFont : Variant;

begin
  IF Active AND (ActiveCellProperty <> []) THEN
  BEGIN
    Range := FSheet.Range[CellName[X1,Y1], CellName[X2,Y2]];

    IF ((cpSetFont IN ActiveCellProperty)
         OR
        (cpSetFontColor IN ActiveCellProperty))
         AND
        Assigned(FFont) THEN
    BEGIN
      ExcelFont := Range.Font;

      IF (cpSetFont IN ActiveCellProperty) THEN
      BEGIN
        ExcelFont.Size  := FFont.Size;
        ExcelFont.Name  := FFont.Name;
        ExcelFont.Bold          := (fsBold      IN FFont.Style);
        ExcelFont.Italic        := (fsItalic    IN FFont.Style);
        ExcelFont.Underline     := (fsUnderline IN FFont.Style);
        ExcelFont.Strikethrough := (fsStrikeOut IN FFont.Style);
      END;

      IF (cpSetFontColor IN ActiveCellProperty)
      THEN ExcelFont.Color := ColorToRGB(FFont.Color);
    END;

    IF (cpSetColor IN ActiveCellProperty)
    THEN Range.Interior.Color := ColorToRGB(FColor);

    IF (cpSetBorderColor IN ActiveCellProperty)
    THEN Range.Borders.Color := ColorToRGB(FBorderColor);
  END;
end; { CellPropertysToArea }

{ ----- }

procedure TGPCustomExcel.CellPropertysFromCell(X,Y: integer);
var
  Range     : Variant;
  ExcelFont : Variant;

BEGIN
  IF Active AND (ActiveCellProperty <> []) THEN
  BEGIN
    Range := FSheet.Range[CellName[X,Y]];

    IF ((cpSetFont IN ActiveCellProperty)
         OR
        (cpSetFontColor IN ActiveCellProperty))
         AND
        Assigned(FFont) THEN
    BEGIN
      ExcelFont := Range.Font;

      IF (cpSetFont IN ActiveCellProperty) THEN
      BEGIN
        FFont.Size := ExcelFont.Size;
        FFont.Name := ExcelFont.Name;
        FFont.Style:= [];
        IF ExcelFont.Bold = true
        THEN FFont.Style := FFont.Style + [fsBold];
        IF ExcelFont.Italic = true
        THEN FFont.Style := FFont.Style + [fsItalic];
        IF ExcelFont.Underline = true
        THEN FFont.Style := FFont.Style + [fsUnderline];
        IF ExcelFont.Strikethrough = true
        THEN FFont.Style := FFont.Style + [fsStrikeOut];
      END;
    END;
  END;
END; { CellPropertysFromCell }

{ ----- }

function TGPCustomExcel.GetActive: boolean;
begin
  Result := FActive;
end; { GetActive }

{ ----- }

function TGPCustomExcel.GetVisible: boolean;
begin
  Result := FVisible;
end; { GetVisible }

{ ----- }

function TGPCustomExcel.GetSetCellProperty: TSetCellPropertys;
begin
  Result := FSetCellProperty;
end; { GetSetCellProperty }

{ ----- }

procedure TGPCustomExcel.SetSetCellProperty(const Value: TSetCellPropertys);
begin
 FSetCellProperty := Value;
end; { SetSetCellProperty }

{ ----- }

function TGPCustomExcel.GetFont: TFont;
begin
  Result := FFont;
end; { GetFont }

{ ----- }

function TGPCustomExcel.GetColor: TColor;
begin
  Result := FColor;
end; { GetColor }

{ ----- }

procedure TGPCustomExcel.SetColor(const Value: TColor);
begin
  FColor := Value;
end; { SetColor }

{ ----- }

function TGPCustomExcel.GetBorderColor: TColor;
begin
  Result := FBorderColor;
end; { GetBorderColor }

{ ----- }

procedure TGPCustomExcel.SetBorderColor(const Value: TColor);
begin
  FBorderColor := Value;
end; { SetBorderColor }

{ ----- }

function TGPCustomExcel.GetCaption: string;
begin
  Result := FCaption;
end; { GetCaption }

{ ----- }

procedure TGPCustomExcel.SetCaption(const Value: string);
begin
  IF Active
  THEN Excel.Caption := Value;
  FCaption := Value;
end; { SetCaption }

{ ----- }

function TGPCustomExcel.GetDefaultSheetName: string;
begin
  Result := FDefaultSheetName;
end; { GetDefaultSheetName }

{ ----- }

procedure TGPCustomExcel.SetDefaultSheetName(const Value: string);
begin
  IF Value <> ''
  THEN FDefaultSheetName := Value;
end; { SetDefaultSheetName }

{ ----- }

procedure TGPCustomExcel.ExternalClose(Sender: TObject;
                                       var Wb, Cancel: OleVariant);
begin
  if Assigned(FOnBeforeClose) then
  begin
    FOnBeforeClose(Sender);
  end;

  Active := false;
end; { ExternalClose }

{ ----- }

function TGPCustomExcel.GetCellWidth(X: integer): integer;
var
  Name : string;

begin
  IF Active THEN
  BEGIN
    Name := CellName[X,Y0];
    SetLength(Name, LENGTH(Name) - 1);
    Result := FSheet.Columns[Name].ColumnWidth;
  END
  ELSE
    Result := -1;
end; { GetCellWidth }

{ ----- }

procedure TGPCustomExcel.SetCellWidth(X: integer; const Value: integer);
VAR
  Name : STRING;

begin
  IF Active THEN
  BEGIN
    Name := CellName[X,Y0];
    SetLength(Name, LENGTH(Name) - 1);
    FSheet.Columns[Name].ColumnWidth := Value;
  END;
end; { SetCellWidth }

{ ----- }

function TGPCustomExcel.GetCellHeight(Y: integer): integer;
begin
  IF Active THEN
  BEGIN
    Result := FSheet.Rows[StrToInt(Copy(CellName[X0,Y], 2,19))].RowHeight;
  END
  ELSE
    Result := -1;
end; { GetCellHeight }

{ ----- }

procedure TGPCustomExcel.SetCellHeight(Y: integer; const Value: integer);
begin
  IF Active THEN
  BEGIN
    FSheet.Rows[StrToInt(Copy(CellName[X0,Y], 2,19))].RowHeight := Value;
  END;
end; { SetCellHeight }

{ ----- }

end.
